tetrimino = class("tetrimino")

function tetrimino:initialize(blockID, rotation, x, y, color)
	self.children = {}

	self.x = x
	self.y = y

	self.graphic = tetriminoGraphics[blockID]

	self.width = self.graphic:getWidth()
	self.height = self.graphic:getHeight()

	self.rotationSpeed = 16

	self.id = blockID

	self.color = color

	self.currentTarget = 1 or rotation

	self.rotationAngles =
	{
		0,
		(3 * math.pi) / 2,
		math.pi,
		math.pi / 2
	}

	self.rotationTarget = self.rotationAngles[self.currentTarget]

	self.rotation = 0 or self.rotationTarget

	self:performLayout(true)

	self.dropTimer = 2
end

function tetrimino:update(dt)
	if self.isRotating then
		local speed, dist = 16, math.min

		self.rotation = dist(self.rotation + speed * dt, (math.pi * 2 ) - self.rotationTarget)

		if self.rotation == (math.pi * 2 ) - self.rotationTarget then
			self.isRotating = false
		end
	end

	if not self.isRotating then
		if self.rotationTarget == 0 then
			self.rotation = 0
		end
	end

	if self.dropTimer > 0 then
		self.dropTimer = self.dropTimer - dt
	else
		self.y = self.y + 16
		self.dropTimer = 2
	end

	for k, v in ipairs(self.children) do
		local x, y = (self.x + v.offsetX), (self.y + v.offsetY)

		v.x = x + math.cos( (math.pi * 2 ) - self.rotationTarget )
		v.y = y + math.sin( (math.pi * 2 ) - self.rotationTarget )
	end
end

function tetrimino:move(dir)
	local speed = 16
	if dir == "left" then
		speed = -16
	end
	self.x = self.x + speed
end

function tetrimino:draw()
	love.graphics.setColor(self.color)

	local offX, offY = 0, 0

	if self.rotation ~= 0 then
		offX, offY = self.width / 2, self.height / 2
	end

	love.graphics.draw(self.graphic, self.x + offX, self.y + offY, self.rotation, scale, scale, offX, offY)

	for k, v in pairs(self.children) do
		v:draw()
	end
end

function tetrimino:performLayout(first)
	if first then
		local imgData = self.graphic:getData()

		for y = 0, imgData:getHeight() - 1 do
			for x = 0, imgData:getWidth() - 1 do
				local r, g, b, a = imgData:getPixel(x, y)

				if a > 0 then
					if y % 16 == 0 and x % 16 == 0 then
						table.insert(self.children, block:new(self.x + x, self.y + y, self.color, x, y))
					end
				end
			end
		end
	end
end

function tetrimino:rotate()
	self.currentTarget = self.currentTarget + 1

	if self.currentTarget > #self.rotationAngles then
		self.currentTarget = 1
	end

	self.rotationTarget = self.rotationAngles[self.currentTarget]

	self.isRotating = true

	self:performLayout()
end